import json


class Request:
    """Express-like request wrapper for Django"""

    def __init__(self, django_request):
        self._request = django_request
        self.method = django_request.method
        self.path = django_request.path
        self.headers = django_request.headers
        # Expose Django request for full access
        self.django = django_request

    @property
    def body(self):
        """Get parsed request body"""
        if self._request.content_type == "application/json":
            try:
                return json.loads(self._request.body)
            except:
                return {}
        return self._request.POST.dict()

    @property
    def query(self):
        """Get query parameters"""
        return self._request.GET.dict()

    @property
    def params(self):
        """Get URL parameters (set by router)"""
        return getattr(self, "_params", {})

    def get(self, key, default=None):
        """Get value from query, body, or params"""
        return (
            self.query.get(key) or self.body.get(key) or self.params.get(key, default)
        )

    @property
    def user(self):
        """Get authenticated user"""
        return self._request.user

    @user.setter
    def user(self, value):
        """Set authenticated user"""
        self._request.user = value

    @property
    def session(self):
        """Get session"""
        return self._request.session

    @property
    def cookies(self):
        """Get cookies"""
        return self._request.COOKIES

    @property
    def files(self):
        """Get uploaded files"""
        return self._request.FILES

    @property
    def META(self):
        """Get request META"""
        return self._request.META



"""ORM wrapper for Shanks Django - Prisma-like syntax"""

from django.db import models
from django.utils.text import slugify as django_slugify


# Re-export Django models with Shanks naming
class Model(models.Model):
    """Base model class with Prisma-like methods"""

    class Meta:
        abstract = True

    @classmethod
    def find_many(cls, **filters):
        """Find many records (Prisma-like)"""
        return cls.objects.filter(**filters)

    @classmethod
    def find_first(cls, **filters):
        """Find first record (Prisma-like)"""
        return cls.objects.filter(**filters).first()

    @classmethod
    def find_unique(cls, **filters):
        """Find unique record (Prisma-like)"""
        try:
            return cls.objects.get(**filters)
        except cls.DoesNotExist:
            return None

    @classmethod
    def create(cls, **data):
        """Create record (Prisma-like)"""
        return cls.objects.create(**data)

    @classmethod
    def update(cls, where, data):
        """Update records (Prisma-like)"""
        return cls.objects.filter(**where).update(**data)

    @classmethod
    def delete_many(cls, **filters):
        """Delete many records (Prisma-like)"""
        return cls.objects.filter(**filters).delete()

    @classmethod
    def count(cls, **filters):
        """Count records (Prisma-like)"""
        if filters:
            return cls.objects.filter(**filters).count()
        return cls.objects.count()

    def update_self(self, **data):
        """Update current instance"""
        for key, value in data.items():
            setattr(self, key, value)
        self.save()
        return self

    def delete_self(self):
        """Delete current instance"""
        self.delete()


# Field types
CharField = models.CharField
TextField = models.TextField
IntegerField = models.IntegerField
FloatField = models.FloatField
BooleanField = models.BooleanField
DateField = models.DateField
DateTimeField = models.DateTimeField
EmailField = models.EmailField
URLField = models.URLField
SlugField = models.SlugField
JSONField = models.JSONField

# Relationship fields
ForeignKey = models.ForeignKey
ManyToManyField = models.ManyToManyField
OneToOneField = models.OneToOneField

# Field options
CASCADE = models.CASCADE
SET_NULL = models.SET_NULL
PROTECT = models.PROTECT


# User helper class with Prisma-like methods (lazy import)
class UserManager:
    """User manager with Prisma-like syntax"""

    @staticmethod
    def _get_user_model():
        """Lazy import Django User"""
        from django.contrib.auth.models import User as DjangoUser

        return DjangoUser

    @staticmethod
    def find_many(**filters):
        """Find many users"""
        return UserManager._get_user_model().objects.filter(**filters)

    @staticmethod
    def find_first(**filters):
        """Find first user"""
        return UserManager._get_user_model().objects.filter(**filters).first()

    @staticmethod
    def find_unique(**filters):
        """Find unique user"""
        DjangoUser = UserManager._get_user_model()
        try:
            return DjangoUser.objects.get(**filters)
        except DjangoUser.DoesNotExist:
            return None

    @staticmethod
    def create(username, email, password, **kwargs):
        """Create user"""
        return UserManager._get_user_model().objects.create_user(
            username=username, email=email, password=password, **kwargs
        )

    @staticmethod
    def count(**filters):
        """Count users"""
        DjangoUser = UserManager._get_user_model()
        if filters:
            return DjangoUser.objects.filter(**filters).count()
        return DjangoUser.objects.count()


# Export User as the manager
User = UserManager


def authenticate(username: str, password: str):
    """Authenticate user"""
    from django.contrib.auth import authenticate as django_authenticate

    return django_authenticate(username=username, password=password)


def create_user(username: str, email: str, password: str, **kwargs):
    """Create new user"""
    from django.contrib.auth.models import User as DjangoUser

    return DjangoUser.objects.create_user(
        username=username, email=email, password=password, **kwargs
    )


# Utility functions
def slugify(text, allow_unicode=False):
    """Convert text to slug (URL-friendly string)"""
    return django_slugify(text, allow_unicode=allow_unicode)



"""
SORM - Shanks ORM
Clean imports for Shanks ORM functionality
"""

from shanks import Model, CharField, TextField, IntegerField, FloatField
from shanks import BooleanField, DateField, DateTimeField, EmailField
from shanks import URLField, SlugField, JSONField
from shanks import ForeignKey, ManyToManyField, OneToOneField
from shanks import CASCADE, SET_NULL, PROTECT
from shanks import User, authenticate
from shanks import model, Schema


# JSON-like model creation
def table(name, schema):
    """
    Create a model with JSON-like syntax

    Example:
        Category = table("Category", {
            "name": "string:100:unique",
            "slug": "slug:unique",
            "description": "text:blank",
            "created_at": "datetime:auto_now_add"
        })
    """
    return model(name, schema)


__all__ = [
    # Base
    "Model",
    "model",
    "Schema",
    "table",
    # Fields
    "CharField",
    "TextField",
    "IntegerField",
    "FloatField",
    "BooleanField",
    "DateField",
    "DateTimeField",
    "EmailField",
    "URLField",
    "SlugField",
    "JSONField",
    # Relations
    "ForeignKey",
    "ManyToManyField",
    "OneToOneField",
    # Cascade options
    "CASCADE",
    "SET_NULL",
    "PROTECT",
    # Auth
    "User",
    "authenticate",
]



"""Shanks CLI commands"""

import os
import subprocess
import sys
import time
from pathlib import Path


def format_code():
    """Format code with black"""
    try:
        import black
    except ImportError:
        print("Installing black...")
        subprocess.check_call([sys.executable, "-m", "pip", "install", "black"])
        import black

    # Get current directory
    cwd = Path.cwd()

    # Format Python files
    print("🎨 Formatting code with black...")
    result = subprocess.run(
        [
            sys.executable,
            "-m",
            "black",
            ".",
            "--exclude",
            "/(\.venv|venv|env|\.git|__pycache__|\.pytest_cache|\.mypy_cache|build|dist|\.eggs)/",
        ],
        cwd=cwd,
    )

    if result.returncode == 0:
        print("✨ Code formatted successfully!")
    else:
        print("❌ Formatting failed")
        sys.exit(1)


def lint_code():
    """Lint code with flake8"""
    try:
        import flake8
    except ImportError:
        print("Installing flake8...")
        subprocess.check_call([sys.executable, "-m", "pip", "install", "flake8"])

    print("🔍 Linting code with flake8...")
    result = subprocess.run(
        [
            sys.executable,
            "-m",
            "flake8",
            ".",
            "--exclude=.venv,venv,env,.git,__pycache__,.pytest_cache,.mypy_cache,build,dist,.eggs",
            "--max-line-length=127",
            "--extend-ignore=E203,W503",
        ]
    )

    if result.returncode == 0:
        print("✅ No linting errors!")
    else:
        print("⚠️  Linting issues found")
        sys.exit(1)


def run_server():
    """Run Django development server with auto-reload (like nodemon)"""
    try:
        import watchdog
    except ImportError:
        print("Installing watchdog for auto-reload...")
        subprocess.check_call([sys.executable, "-m", "pip", "install", "watchdog"])

    # Check if manage.py exists
    if not Path("manage.py").exists():
        print("❌ manage.py not found!")
        print("Make sure you're in a Django project directory.")
        sys.exit(1)

    # Get host and port from args
    host = "127.0.0.1"
    port = "8000"

    if len(sys.argv) > 2:
        for arg in sys.argv[2:]:
            if ":" in arg:
                host, port = arg.split(":")
            elif arg.isdigit():
                port = arg

    print(f"🚀 Starting Shanks development server...")
    print(f"📡 Server running at http://{host}:{port}")
    print(f"🔄 Auto-reload enabled (like nodemon)")
    print(f"⏹️  Press Ctrl+C to stop\n")

    # Run Django development server with auto-reload
    try:
        subprocess.run(
            [sys.executable, "manage.py", "runserver", f"{host}:{port}"],
            check=True,
        )
    except KeyboardInterrupt:
        print("\n\n⏹️  Server stopped")
    except subprocess.CalledProcessError as e:
        print(f"\n❌ Server error: {e}")
        sys.exit(1)


def create_project():
    """Create a new Shanks Django project with Go-like architecture"""
    if len(sys.argv) < 3:
        print("Usage: shanks new <project_name>")
        sys.exit(1)

    project_name = sys.argv[2]
    print(f"🎉 Creating new Shanks project: {project_name}")

    # Create Django project
    try:
        subprocess.run(
            [sys.executable, "-m", "django", "startproject", project_name], check=True
        )
    except (FileNotFoundError, subprocess.CalledProcessError):
        print("❌ Django not found. Installing Django...")
        subprocess.check_call([sys.executable, "-m", "pip", "install", "django"])
        subprocess.run(
            [sys.executable, "-m", "django", "startproject", project_name], check=True
        )

    project_dir = Path(project_name)
    config_dir = project_dir / project_name

    # Create Go-like architecture
    internal_dir = project_dir / "internal"
    internal_dir.mkdir(exist_ok=True)
    (internal_dir / "__init__.py").write_text("")

    # Create internal subdirectories
    for subdir in ["controller", "repository", "service", "middleware", "routes"]:
        dir_path = internal_dir / subdir
        dir_path.mkdir(exist_ok=True)
        (dir_path / "__init__.py").write_text("")

    # Create other directories
    for dir_name in ["dto", "entity", "config", "utils"]:
        dir_path = project_dir / dir_name
        dir_path.mkdir(exist_ok=True)
        (dir_path / "__init__.py").write_text("")

    # Create routes with example
    routes_content = '''"""API Routes"""
from shanks import App, swagger, auto_cache, smart_cache_invalidation

app = App()

# Enable built-in caching - auto-cache GET requests!
app.use(auto_cache)

# Smart cache invalidation - auto-clear cache on POST/PUT/DELETE
app.use(smart_cache_invalidation)

# Enable Swagger - applies to all endpoints!
app.use(swagger(title="{project_name} API", version="1.0.0"))


@app.get("api/health")
def health(req):
    """Health check"""
    return {{"status": "ok", "service": "{project_name}"}}


# Export URL patterns
urlpatterns = app.get_urls()
'''.format(project_name=project_name)
    (internal_dir / "routes" / "__init__.py").write_text(routes_content)

    # Create example middleware
    middleware_content = '''"""Middleware"""


def logger(req, res, next):
    """Log all requests - Express.js style"""
    print(f"[{{req.method}}] {{req.path}}")
    next()
'''
    (internal_dir / "middleware" / "logger.py").write_text(middleware_content)

    # Update settings.py
    settings_content = '''"""Shanks Django Settings"""

from shanks import (
    get_base_dir,
    get_secret_key,
    get_debug,
    get_allowed_hosts,
    get_database,
    get_installed_apps,
    get_middleware,
    get_templates,
    get_password_validators,
)

# Paths
BASE_DIR = get_base_dir(__file__)

# Security
SECRET_KEY = get_secret_key()
DEBUG = get_debug()
ALLOWED_HOSTS = get_allowed_hosts()

# Apps
INSTALLED_APPS = get_installed_apps(["internal", "entity", "dto"])

# Middleware
MIDDLEWARE = get_middleware()

# Routing
ROOT_URLCONF = "internal.routes"

# Templates
TEMPLATES = get_templates()

# WSGI
WSGI_APPLICATION = "{project_name}.wsgi.application"

# Database
DATABASES = get_database(BASE_DIR)

# Password validation
AUTH_PASSWORD_VALIDATORS = get_password_validators(DEBUG)

# Internationalization
LANGUAGE_CODE = "en-us"
TIME_ZONE = "UTC"
USE_I18N = True
USE_TZ = True

# Static & Media
STATIC_URL = "static/"
STATIC_ROOT = BASE_DIR / "staticfiles"
MEDIA_URL = "media/"
MEDIA_ROOT = BASE_DIR / "media"

# Default
DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"
'''.format(project_name=project_name)
    (config_dir / "settings.py").write_text(settings_content)

    # Update wsgi.py
    wsgi_content = '''"""WSGI"""

import os
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "{project_name}.settings")

from django.core.wsgi import get_wsgi_application
application = get_wsgi_application()
'''.format(project_name=project_name)
    (config_dir / "wsgi.py").write_text(wsgi_content)

    # Delete urls.py
    urls_file = config_dir / "urls.py"
    if urls_file.exists():
        urls_file.unlink()

    # Override manage.py to suppress warnings
    manage_content = '''#!/usr/bin/env python
"""Django's command-line utility for administrative tasks."""
import os
import sys
import warnings


def main():
    """Run administrative tasks."""
    # Suppress sys.prefix warnings
    warnings.filterwarnings("ignore", category=RuntimeWarning, module="site")
    
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', '{project_name}.settings')
    try:
        from django.core.management import execute_from_command_line
    except ImportError as exc:
        raise ImportError(
            "Couldn't import Django. Are you sure it's installed and "
            "available on your PYTHONPATH environment variable? Did you "
            "forget to activate a virtual environment?"
        ) from exc
    execute_from_command_line(sys.argv)


if __name__ == '__main__':
    main()
'''.format(project_name=project_name)
    (project_dir / "manage.py").write_text(manage_content)

    # Create .env.example
    env_example = """# Django Settings
SECRET_KEY=your-secret-key-here
DEBUG=True
ALLOWED_HOSTS=*

# Database (optional)
# DATABASE_URL=postgresql://user:pass@localhost/dbname
"""
    (project_dir / ".env.example").write_text(env_example)

    # Create README.md
    readme_content = """# {project_name}

Shanks Django project with Go-like architecture

## Structure

```
{project_name}/
├── internal/           # Internal application code
│   ├── controller/    # HTTP handlers
│   ├── repository/    # Data access layer
│   ├── service/       # Business logic
│   ├── middleware/    # Middleware functions
│   └── routes/        # Route definitions
├── entity/            # Database models
├── dto/               # Data Transfer Objects
├── config/            # Configuration files
├── utils/             # Utility functions
└── {project_name}/    # Django settings
```

## Quick Start

```bash
# Install dependencies
pip install shanks-django

# Copy environment file
cp .env.example .env

# Run migrations
python manage.py migrate

# Start server
python manage.py runserver
```

Visit: http://127.0.0.1:8000/api/health
Swagger: http://127.0.0.1:8000/docs

## Generate CRUD

```bash
shanks create posts --crud
shanks create auth --simple
```
""".format(project_name=project_name)
    (project_dir / "README.md").write_text(readme_content)

    print(f"\n✅ Project created successfully!")
    print(f"\n📁 Go-like architecture:")
    print(f"  {project_name}/")
    print(f"  ├── internal/")
    print(f"  │   ├── controller/    # HTTP handlers")
    print(f"  │   ├── repository/    # Data access")
    print(f"  │   ├── service/       # Business logic")
    print(f"  │   ├── middleware/    # Middleware")
    print(f"  │   └── routes/        # Routes")
    print(f"  ├── entity/            # Models")
    print(f"  ├── dto/               # DTOs")
    print(f"  ├── config/            # Config")
    print(f"  └── utils/             # Utils")
    print(f"\n🚀 Next steps:")
    print(f"  cd {project_name}")
    print(f"  cp .env.example .env")
    print(f"  python manage.py migrate")
    print(f"  python manage.py runserver")
    print(f"\n🌐 Visit: http://127.0.0.1:8000/api/health")
    print(f"\n📚 Swagger: http://127.0.0.1:8000/docs")


def create_crud_endpoint():
    """Create CRUD endpoint with Go-like architecture"""
    if len(sys.argv) < 3:
        print("Usage: shanks create <endpoint_name> --crud")
        print("Example: shanks create posts --crud")
        sys.exit(1)

    endpoint_name = sys.argv[2]
    model_name = endpoint_name.capitalize().rstrip("s")
    endpoint_plural = endpoint_name.lower()

    if not Path("manage.py").exists():
        print("❌ Not in a Shanks project directory!")
        sys.exit(1)

    # Create entity (model) - use Django models directly
    entity_dir = Path("entity")
    entity_dir.mkdir(exist_ok=True)
    if not (entity_dir / "__init__.py").exists():
        (entity_dir / "__init__.py").write_text("")

    entity_file = entity_dir / f"{endpoint_name}.py"
    entity_content = f'''"""
{model_name} Entity
"""
from django.db import models
from django.contrib.auth.models import User


class {model_name}(models.Model):
    """
{model_name} model
    """
    title = models.CharField(max_length=200)
    description = models.TextField(blank=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    created_by = models.ForeignKey(User, on_delete=models.CASCADE, related_name="{endpoint_plural}")

    class Meta:
        app_label = "entity"
        ordering = ["-created_at"]

    def __str__(self):
        return self.title

    # Prisma-like methods
    @classmethod
    def find_many(cls, **filters):
        return cls.objects.filter(**filters)

    @classmethod
    def find_unique(cls, **filters):
        try:
            return cls.objects.get(**filters)
        except cls.DoesNotExist:
            return None

    @classmethod
    def count(cls, **filters):
        if filters:
            return cls.objects.filter(**filters).count()
        return cls.objects.count()

    def update_self(self, **data):
        for key, value in data.items():
            setattr(self, key, value)
        self.save()
        return self

    def delete_self(self):
        self.delete()
'''
    entity_file.write_text(entity_content)

    # Update entity/models.py for Django auto-discovery
    models_file = entity_dir / "models.py"
    if models_file.exists():
        models_content = models_file.read_text()
        import_line = f"from .{endpoint_name} import {model_name}"
        if import_line not in models_content:
            models_content += f"\n{import_line}\n"
            models_file.write_text(models_content)
    else:
        models_file.write_text(f"from .{endpoint_name} import {model_name}\n")

    # Create controller
    controller_dir = Path("internal/controller")
    controller_file = controller_dir / f"{endpoint_name}.py"
    controller_content = f'''"""
{model_name} Controller
"""
from shanks import Response
from entity.{endpoint_name} import {model_name}


def list_{endpoint_plural}(req):
    """List with pagination"""
    page = int(req.query.get("page", 1))
    limit = int(req.query.get("limit", 10))
    offset = (page - 1) * limit

    total = {model_name}.count()
    items = {model_name}.find_many()[offset : offset + limit]

    return {{
        "data": [
            {{
                "id": item.id,
                "title": item.title,
                "description": item.description,
                "created_at": item.created_at.isoformat(),
            }}
            for item in items
        ],
        "pagination": {{"page": page, "limit": limit, "total": total}},
    }}


def get_by_id(req, id):
    """Get by ID"""
    item = {model_name}.find_unique(id=id)
    if not item:
        return Response().status_code(404).json({{"error": "Not found"}})
    
    return {{
        "id": item.id,
        "title": item.title,
        "description": item.description,
        "created_at": item.created_at.isoformat(),
    }}


def create(req):
    """Create"""
    if not req.user.is_authenticated:
        return Response().status_code(401).json({{"error": "Auth required"}})
    
    data = req.body
    item = {model_name}.objects.create(
        title=data.get("title"),
        description=data.get("description", ""),
        created_by=req.user,
    )
    return Response().status_code(201).json({{"id": item.id}})


def update(req, id):
    """Update"""
    if not req.user.is_authenticated:
        return Response().status_code(401).json({{"error": "Auth required"}})
    
    item = {model_name}.find_unique(id=id)
    if not item:
        return Response().status_code(404).json({{"error": "Not found"}})
    
    data = req.body
    item.update_self(
        title=data.get("title", item.title),
        description=data.get("description", item.description),
    )
    return {{"message": "Updated"}}


def delete(req, id):
    """Delete"""
    if not req.user.is_authenticated:
        return Response().status_code(401).json({{"error": "Auth required"}})
    
    item = {model_name}.find_unique(id=id)
    if not item:
        return Response().status_code(404).json({{"error": "Not found"}})
    
    item.delete_self()
    return {{"message": "Deleted"}}
'''
    controller_file.write_text(controller_content)

    # Create routes
    routes_dir = Path("internal/routes")
    routes_file = routes_dir / f"{endpoint_name}.py"
    routes_content = f'''"""
{model_name} Routes
"""
from shanks import App
from internal.controller import {endpoint_name} as ctrl

router = App()


@router.get("api/{endpoint_plural}")
def list_{endpoint_plural}(req):
    return ctrl.list_{endpoint_plural}(req)


@router.get("api/{endpoint_plural}/<int:id>")
def get_{endpoint_name}(req, id):
    return ctrl.get_by_id(req, id)


@router.post("api/{endpoint_plural}")
def create_{endpoint_name}(req):
    return ctrl.create(req)


@router.put("api/{endpoint_plural}/<int:id>")
def update_{endpoint_name}(req, id):
    return ctrl.update(req, id)


@router.delete("api/{endpoint_plural}/<int:id>")
def delete_{endpoint_name}(req, id):
    return ctrl.delete(req, id)
'''
    routes_file.write_text(routes_content)

    print(f"\n✅ CRUD created!")
    print(f"\n📁 Files:")
    print(f"  ├── entity/{endpoint_name}.py")
    print(f"  ├── internal/controller/{endpoint_name}.py")
    print(f"  └── internal/routes/{endpoint_name}.py")
    print(f"\n📝 Next steps:")
    print(f"   1. Import in internal/routes/__init__.py:")
    print(f"      from . import {endpoint_name}")
    print(f"      app.include({endpoint_name}.router)")
    print(f"   2. Run migrations:")
    print(f"      python manage.py makemigrations")
    print(f"      python manage.py migrate")
    print(f"\n🎯 Endpoints:")
    print(f"  GET    /api/{endpoint_plural}")
    print(f"  GET    /api/{endpoint_plural}/<id>")
    print(f"  POST   /api/{endpoint_plural}")
    print(f"  PUT    /api/{endpoint_plural}/<id>")
    print(f"  DELETE /api/{endpoint_plural}/<id>")


def create_auth_endpoint():
    """Create authentication endpoints"""
    if len(sys.argv) < 4:
        print("Usage: shanks create auth --simple|--complete")
        sys.exit(1)

    auth_type = sys.argv[3]
    if auth_type not in ["--simple", "--complete"]:
        print("❌ Use --simple or --complete")
        sys.exit(1)

    if not Path("manage.py").exists():
        print("❌ Not in a Shanks project!")
        sys.exit(1)

    routes_dir = Path("internal/routes")
    auth_file = routes_dir / "auth.py"

    if auth_type == "--simple":
        auth_content = '''"""Auth Routes - Simple"""
from shanks import App, Response
from SORM import User, authenticate

router = App()


@router.post("api/auth/register")
def register(req):
    data = req.body
    username = data.get("username")
    email = data.get("email")
    password = data.get("password")

    if not username or not email or not password:
        return Response().status_code(400).json({"error": "All fields required"})

    if User.find_unique(username=username):
        return Response().status_code(400).json({"error": "Username exists"})

    user = User.create(username=username, email=email, password=password)
    return Response().status_code(201).json({"message": "Registered", "user_id": user.id})


@router.post("api/auth/login")
def login(req):
    data = req.body
    user = authenticate(username=data.get("username"), password=data.get("password"))
    
    if not user:
        return Response().status_code(401).json({"error": "Invalid credentials"})
    
    return {"message": "Login successful", "user_id": user.id}


@router.get("api/auth/me")
def get_me(req):
    if not req.user.is_authenticated:
        return Response().status_code(401).json({"error": "Not authenticated"})
    
    return {"id": req.user.id, "username": req.user.username, "email": req.user.email}
'''
    else:  # --complete
        auth_content = '''"""Auth Routes - Complete"""
from shanks import App, Response
from SORM import User, authenticate

router = App()


@router.post("api/auth/register")
def register(req):
    data = req.body
    username = data.get("username")
    email = data.get("email")
    password = data.get("password")

    if not username or not email or not password:
        return Response().status_code(400).json({"error": "All fields required"})

    if User.find_unique(username=username):
        return Response().status_code(400).json({"error": "Username exists"})

    user = User.create(username=username, email=email, password=password, is_active=False)
    # TODO: Send verification email
    return Response().status_code(201).json({"message": "Check email to verify"})


@router.post("api/auth/verify")
def verify_email(req):
    token = req.body.get("token")
    # TODO: Implement verification
    return {"message": "Email verified"}


@router.post("api/auth/login")
def login(req):
    data = req.body
    user = authenticate(username=data.get("username"), password=data.get("password"))
    
    if not user:
        return Response().status_code(401).json({"error": "Invalid credentials"})
    
    if not user.is_active:
        return Response().status_code(403).json({"error": "Account not verified"})
    
    return {"message": "Login successful", "user_id": user.id}


@router.get("api/auth/me")
def get_me(req):
    if not req.user.is_authenticated:
        return Response().status_code(401).json({"error": "Not authenticated"})
    
    return {"id": req.user.id, "username": req.user.username, "email": req.user.email}
'''

    auth_file.write_text(auth_content)

    endpoints = ["POST /api/auth/register", "POST /api/auth/login", "GET  /api/auth/me"]
    if auth_type == "--complete":
        endpoints.insert(2, "POST /api/auth/verify")

    print(f"\n✅ Auth created!")
    print(f"\n📁 File: internal/routes/auth.py")
    print(f"\n📝 Import in internal/routes/__init__.py:")
    print(f"   from . import auth")
    print(f"   app.include(auth.router)")
    print(f"\n🎯 Endpoints:")
    for e in endpoints:
        print(f"  {e}")


def show_help():
    """Show help message"""
    help_text = """
Shanks Django CLI

Usage:
    shanks <command> [options]

Commands:
    run [host:port]              Run development server with auto-reload
                                 Default: 127.0.0.1:8000

    new <name>                   Create a new Shanks Django project
                                 with Go-like architecture

    create <endpoint> --crud     Generate CRUD endpoints with model
                                 Creates: entity, controller, routes
                                 Example: shanks create products --crud

    create auth --simple         Generate simple auth endpoints
                                 Creates: /login, /register, /me

    create auth --complete       Generate complete auth endpoints
                                 Creates: /login, /register, /verify, /me

    format                       Format code with black
    lint                         Lint code with flake8
    check                        Format and lint code

    help                         Show this help message

Examples:
    shanks new myproject                # Create new project
    shanks create products --crud       # Generate products CRUD
    shanks create auth --simple         # Generate auth endpoints
    shanks run                          # Start server
    shanks format                       # Format code
"""
    print(help_text)


def main():
    """Main CLI entry point"""
    if len(sys.argv) < 2:
        show_help()
        sys.exit(0)

    command = sys.argv[1]

    if command == "run":
        run_server()
    elif command == "new":
        create_project()
    elif command == "create":
        # Handle create subcommands
        if len(sys.argv) < 3:
            print("Usage: shanks create <endpoint> --crud")
            print("       shanks create auth --simple|--complete")
            sys.exit(1)

        subcommand = sys.argv[2]
        if subcommand == "auth":
            create_auth_endpoint()
        elif "--crud" in sys.argv:
            create_crud_endpoint()
        else:
            print(f"Unknown create command: {subcommand}")
            print("Use: shanks create <endpoint> --crud")
            print("Or:  shanks create auth --simple|--complete")
            sys.exit(1)
    elif command == "format":
        format_code()
    elif command == "lint":
        lint_code()
    elif command == "check":
        format_code()
        lint_code()
    elif command == "help":
        show_help()
    else:
        print(f"Unknown command: {command}")
        show_help()
        sys.exit(1)


if __name__ == "__main__":
    main()



"""Database connection helpers for Shanks Django"""

import os
from typing import Any, Dict, Optional


class DatabaseConfig:
    """Database configuration helper"""

    @staticmethod
    def postgres(
        host: str = "localhost",
        port: int = 5432,
        database: str = "postgres",
        user: str = "postgres",
        password: str = "",
        **options,
    ) -> Dict[str, Any]:
        """
        Generate PostgreSQL database configuration

        Example:
            DATABASES = {
                'default': DatabaseConfig.postgres(
                    host='localhost',
                    database='mydb',
                    user='myuser',
                    password='mypass'
                )
            }
        """
        config = {
            "ENGINE": "django.db.backends.postgresql",
            "NAME": database,
            "USER": user,
            "PASSWORD": password,
            "HOST": host,
            "PORT": port,
        }
        if options:
            config["OPTIONS"] = options
        return config

    @staticmethod
    def mysql(
        host: str = "localhost",
        port: int = 3306,
        database: str = "mysql",
        user: str = "root",
        password: str = "",
        **options,
    ) -> Dict[str, Any]:
        """
        Generate MySQL database configuration

        Example:
            DATABASES = {
                'default': DatabaseConfig.mysql(
                    host='localhost',
                    database='mydb',
                    user='myuser',
                    password='mypass'
                )
            }
        """
        config = {
            "ENGINE": "django.db.backends.mysql",
            "NAME": database,
            "USER": user,
            "PASSWORD": password,
            "HOST": host,
            "PORT": port,
        }
        if options:
            config["OPTIONS"] = options
        return config

    @staticmethod
    def sqlite(path: str = "db.sqlite3") -> Dict[str, Any]:
        """
        Generate SQLite database configuration

        Example:
            DATABASES = {
                'default': DatabaseConfig.sqlite('db.sqlite3')
            }
        """
        return {
            "ENGINE": "django.db.backends.sqlite3",
            "NAME": path,
        }

    @staticmethod
    def from_url(url: str) -> Dict[str, Any]:
        """
        Parse database URL and generate configuration

        Supports:
        - postgresql://user:pass@host:port/dbname
        - mysql://user:pass@host:port/dbname
        - sqlite:///path/to/db.sqlite3

        Example:
            DATABASES = {
                'default': DatabaseConfig.from_url(os.getenv('DATABASE_URL'))
            }
        """
        try:
            import dj_database_url

            return dj_database_url.parse(url)
        except ImportError:
            # Fallback manual parsing
            if url.startswith("sqlite"):
                path = url.replace("sqlite:///", "")
                return DatabaseConfig.sqlite(path)

            # Basic parsing for postgres/mysql
            if "://" in url:
                scheme, rest = url.split("://", 1)
                if "@" in rest:
                    auth, location = rest.split("@", 1)
                    user, password = auth.split(":", 1) if ":" in auth else (auth, "")
                    host_port, database = (
                        location.split("/", 1) if "/" in location else (location, "")
                    )
                    host, port = (
                        host_port.split(":", 1)
                        if ":" in host_port
                        else (host_port, None)
                    )

                    if scheme == "postgresql" or scheme == "postgres":
                        return DatabaseConfig.postgres(
                            host=host,
                            port=int(port) if port else 5432,
                            database=database,
                            user=user,
                            password=password,
                        )
                    elif scheme == "mysql":
                        return DatabaseConfig.mysql(
                            host=host,
                            port=int(port) if port else 3306,
                            database=database,
                            user=user,
                            password=password,
                        )

            raise ValueError(f"Unsupported database URL: {url}")


class MongoDB:
    """MongoDB connection helper"""

    _client = None
    _db = None

    @classmethod
    def connect(
        cls,
        host: str = "localhost",
        port: int = 27017,
        database: str = "test",
        username: Optional[str] = None,
        password: Optional[str] = None,
        **kwargs,
    ):
        """
        Connect to MongoDB

        Example:
            from shanks.db import MongoDB

            MongoDB.connect(
                host='localhost',
                database='mydb',
                username='user',
                password='pass'
            )

            # Use in views
            @app.get('api/users')
            def get_users(req):
                users = MongoDB.db.users.find()
                return {'users': list(users)}
        """
        try:
            from pymongo import MongoClient
        except ImportError:
            raise ImportError(
                "pymongo is required for MongoDB support. "
                "Install it with: pip install pymongo"
            )

        if username and password:
            uri = f"mongodb://{username}:{password}@{host}:{port}/{database}"
        else:
            uri = f"mongodb://{host}:{port}/{database}"

        cls._client = MongoClient(uri, **kwargs)
        cls._db = cls._client[database]
        return cls._db

    @classmethod
    def connect_uri(cls, uri: str, database: str):
        """
        Connect to MongoDB using URI

        Example:
            MongoDB.connect_uri(
                'mongodb://user:pass@host:27017',
                'mydb'
            )
        """
        try:
            from pymongo import MongoClient
        except ImportError:
            raise ImportError(
                "pymongo is required for MongoDB support. "
                "Install it with: pip install pymongo"
            )

        cls._client = MongoClient(uri)
        cls._db = cls._client[database]
        return cls._db

    @classmethod
    @property
    def client(cls):
        """Get MongoDB client"""
        if cls._client is None:
            raise RuntimeError("MongoDB not connected. Call MongoDB.connect() first.")
        return cls._client

    @classmethod
    @property
    def db(cls):
        """Get MongoDB database"""
        if cls._db is None:
            raise RuntimeError("MongoDB not connected. Call MongoDB.connect() first.")
        return cls._db


class Redis:
    """Redis connection helper"""

    _client = None

    @classmethod
    def connect(
        cls,
        host: str = "localhost",
        port: int = 6379,
        db: int = 0,
        password: Optional[str] = None,
        **kwargs,
    ):
        """
        Connect to Redis

        Example:
            from shanks.db import Redis

            Redis.connect(
                host='localhost',
                password='mypass'
            )

            # Use in views
            @app.get('api/cache')
            def get_cache(req):
                value = Redis.client.get('key')
                return {'value': value}
        """
        try:
            import redis
        except ImportError:
            raise ImportError(
                "redis is required for Redis support. "
                "Install it with: pip install redis"
            )

        cls._client = redis.Redis(
            host=host,
            port=port,
            db=db,
            password=password,
            decode_responses=True,
            **kwargs,
        )
        return cls._client

    @classmethod
    def connect_url(cls, url: str):
        """
        Connect to Redis using URL

        Example:
            Redis.connect_url('redis://localhost:6379/0')
            Redis.connect_url('redis://:password@localhost:6379/0')
        """
        try:
            import redis
        except ImportError:
            raise ImportError(
                "redis is required for Redis support. "
                "Install it with: pip install redis"
            )

        cls._client = redis.from_url(url, decode_responses=True)
        return cls._client

    @classmethod
    @property
    def client(cls):
        """Get Redis client"""
        if cls._client is None:
            raise RuntimeError("Redis not connected. Call Redis.connect() first.")
        return cls._client


# Convenience functions
def setup_postgres(**kwargs) -> Dict[str, Any]:
    """Shortcut for PostgreSQL configuration"""
    return DatabaseConfig.postgres(**kwargs)


def setup_mysql(**kwargs) -> Dict[str, Any]:
    """Shortcut for MySQL configuration"""
    return DatabaseConfig.mysql(**kwargs)


def setup_sqlite(path: str = "db.sqlite3") -> Dict[str, Any]:
    """Shortcut for SQLite configuration"""
    return DatabaseConfig.sqlite(path)


def setup_mongodb(**kwargs):
    """Shortcut for MongoDB connection"""
    return MongoDB.connect(**kwargs)


def setup_redis(**kwargs):
    """Shortcut for Redis connection"""
    return Redis.connect(**kwargs)



from functools import wraps
from typing import Callable, List

from django.http import JsonResponse
from django.urls import path

from .request import Request
from .response import Response


class App:
    def __init__(self, prefix: str = ""):
        self.routes = []
        self.middlewares = []
        self.prefix = prefix.rstrip("/")

    def use(self, middleware: Callable):
        """Add middleware to the app"""
        # Check if it's a swagger middleware
        if hasattr(middleware, "_is_swagger") and middleware._is_swagger:
            # Enable Swagger with the config
            from .swagger import SwaggerUI

            config = middleware._swagger_config
            SwaggerUI.enable(
                self,
                title=config["title"],
                version=config["version"],
                description=config["description"],
                docs_url=config["docs_url"],
            )
        else:
            self.middlewares.append(middleware)
        return self

    def _create_view(self, handler: Callable, method: str):
        """Create Django view from handler"""

        @wraps(handler)
        def view(request, *args, **kwargs):
            # Wrap Django request
            app_request = Request(request)
            app_response = Response()

            # Store reference for CORS
            request._shanks_request = app_request

            # Middleware chain
            middleware_index = [0]
            handler_called = [False]

            def next_middleware():
                """Call next middleware in chain"""
                if middleware_index[0] < len(self.middlewares):
                    current = self.middlewares[middleware_index[0]]
                    middleware_index[0] += 1

                    # Call middleware with (req, res, next)
                    import inspect

                    sig = inspect.signature(current)
                    param_count = len(sig.parameters)

                    if param_count == 3:
                        # Express.js style: (req, res, next)
                        result = current(app_request, app_response, next_middleware)
                    elif param_count == 1:
                        # Legacy style: (req)
                        result = current(app_request)
                        if not result:
                            next_middleware()
                    else:
                        # Default: call with req
                        result = current(app_request)
                        if not result:
                            next_middleware()

                    return result
                else:
                    # All middlewares done, call handler
                    if not handler_called[0]:
                        handler_called[0] = True
                        return handler(app_request, *args, **kwargs)

            # Start middleware chain
            result = next_middleware()

            # If middleware returned a response, use it
            if result:
                if isinstance(result, Response):
                    return result.to_django_response(request)
                elif isinstance(result, dict):
                    return JsonResponse(result)
                return result

            # Otherwise use handler response
            if handler_called[0]:
                response = handler(app_request, *args, **kwargs)
                if isinstance(response, Response):
                    return response.to_django_response(request)
                elif isinstance(response, dict):
                    return JsonResponse(response)
                return response

            # Fallback
            return JsonResponse({"error": "No response"}, status=500)

        return view

    def get(self, route: str):
        """Decorator for GET routes"""

        def decorator(handler):
            full_path = f"{self.prefix}/{route}".strip("/")
            self.routes.append(
                {
                    "path": full_path,
                    "view": self._create_view(handler, "GET"),
                    "name": handler.__name__,
                }
            )
            return handler

        return decorator

    def post(self, route: str):
        """Decorator for POST routes"""

        def decorator(handler):
            full_path = f"{self.prefix}/{route}".strip("/")
            self.routes.append(
                {
                    "path": full_path,
                    "view": self._create_view(handler, "POST"),
                    "name": handler.__name__,
                }
            )
            return handler

        return decorator

    def put(self, route: str):
        """Decorator for PUT routes"""

        def decorator(handler):
            full_path = f"{self.prefix}/{route}".strip("/")
            self.routes.append(
                {
                    "path": full_path,
                    "view": self._create_view(handler, "PUT"),
                    "name": handler.__name__,
                }
            )
            return handler

        return decorator

    def delete(self, route: str):
        """Decorator for DELETE routes"""

        def decorator(handler):
            full_path = f"{self.prefix}/{route}".strip("/")
            self.routes.append(
                {
                    "path": full_path,
                    "view": self._create_view(handler, "DELETE"),
                    "name": handler.__name__,
                }
            )
            return handler

        return decorator

    def group(self, prefix: str):
        """Create a route group with prefix (like Gin)"""
        return App(prefix=f"{self.prefix}/{prefix}".strip("/"))

    def include(self, *apps):
        """Include routes from other App instances"""
        for app in apps:
            # Copy routes from other app
            for route in app.routes:
                self.routes.append(route)
            # Copy middlewares
            for middleware in app.middlewares:
                if middleware not in self.middlewares:
                    self.middlewares.append(middleware)
        return self

    def get_urls(self):
        """Get Django URL patterns"""
        from django.http import HttpResponse
        from django.shortcuts import render
        from django.template.loader import get_template
        from django.template import TemplateDoesNotExist

        # Check if user has defined root path
        has_root = any(
            route["path"] == "" or route["path"] == "/" for route in self.routes
        )

        patterns = [
            path(route["path"].lstrip("/"), route["view"], name=route["name"])
            for route in self.routes
        ]

        # Add default landing page if no root path defined
        if not has_root:

            def default_landing(request):
                # Try to render user's index.html first
                try:
                    get_template("index.html")
                    return render(request, "index.html")
                except TemplateDoesNotExist:
                    # Render Shanks default landing page
                    import os
                    from pathlib import Path

                    landing_path = Path(__file__).parent / "templates" / "landing.html"
                    with open(landing_path, "r") as f:
                        html = f.read()
                    return HttpResponse(html)

            patterns.insert(0, path("", default_landing, name="shanks_landing"))

        return patterns



"""Built-in Swagger/OpenAPI support for Shanks Django"""

import json
from typing import Any, Dict, List, Optional


class SwaggerUI:
    """
    Built-in Swagger UI for Shanks Django

    Usage:
        from shanks import App, SwaggerUI

        app = App()

        # Enable Swagger UI
        SwaggerUI.enable(app,
            title="My API",
            version="1.0.0",
            description="API documentation"
        )

        # Add documented routes
        @app.get('api/users')
        @SwaggerUI.doc(
            summary="Get all users",
            description="Returns a list of all users",
            responses={200: {"description": "Success"}}
        )
        def get_users(req):
            return {'users': []}
    """

    _spec = None
    _routes = []

    @classmethod
    def enable(
        cls,
        app,
        title: str = "API Documentation",
        version: str = "1.0.0",
        description: str = "",
        docs_url: str = "docs",
        openapi_url: str = "openapi.json",
    ):
        """
        Enable Swagger UI for the app

        Args:
            app: Shanks App instance
            title: API title
            version: API version
            description: API description
            docs_url: URL path for Swagger UI (default: /docs)
            openapi_url: URL path for OpenAPI spec (default: /openapi.json)
        """
        cls._spec = {
            "openapi": "3.0.0",
            "info": {"title": title, "version": version, "description": description},
            "paths": {},
            "components": {"schemas": {}},
        }

        # Add OpenAPI JSON endpoint
        @app.get(openapi_url)
        def openapi_spec(req):
            return cls._generate_spec(app)

        # Add Swagger UI endpoint
        @app.get(docs_url)
        def swagger_ui(req):
            from django.http import HttpResponse

            html = cls._generate_swagger_html(f"/{openapi_url}")
            return HttpResponse(html, content_type="text/html")

    @classmethod
    def doc(
        cls,
        summary: str = "",
        description: str = "",
        tags: Optional[List[str]] = None,
        responses: Optional[Dict[int, Dict[str, Any]]] = None,
        parameters: Optional[List[Dict[str, Any]]] = None,
        request_body: Optional[Dict[str, Any]] = None,
    ):
        """
        Decorator to add documentation to a route

        Example:
            @app.get('api/users/<int:user_id>')
            @SwaggerUI.doc(
                summary="Get user by ID",
                description="Returns a single user",
                parameters=[{
                    "name": "user_id",
                    "in": "path",
                    "required": True,
                    "schema": {"type": "integer"}
                }],
                responses={
                    200: {"description": "Success"},
                    404: {"description": "User not found"}
                }
            )
            def get_user(req, user_id):
                return {'id': user_id}
        """

        def decorator(func):
            func._swagger_doc = {
                "summary": summary,
                "description": description,
                "tags": tags or [],
                "responses": responses or {},
                "parameters": parameters or [],
                "requestBody": request_body,
            }
            return func

        return decorator

    @classmethod
    def _generate_spec(cls, app) -> Dict[str, Any]:
        """Generate OpenAPI specification from routes"""
        spec = cls._spec.copy()

        for route in app.routes:
            path = "/" + route["path"]
            # Convert Django URL patterns to OpenAPI format
            path = path.replace("<int:", "{").replace("<str:", "{").replace(">", "}")

            if path not in spec["paths"]:
                spec["paths"][path] = {}

            # Get method from route
            method = "get"  # Default
            if hasattr(route["view"], "__name__"):
                view_name = route["view"].__name__.lower()
                if "post" in view_name:
                    method = "post"
                elif "put" in view_name:
                    method = "put"
                elif "delete" in view_name:
                    method = "delete"

            # Get documentation from decorator
            doc = {}
            if hasattr(route["view"], "_swagger_doc"):
                doc = route["view"]._swagger_doc

            spec["paths"][path][method] = {
                "summary": doc.get("summary", route["name"]),
                "description": doc.get("description", ""),
                "tags": doc.get("tags", []),
                "parameters": doc.get("parameters", []),
                "responses": doc.get("responses", {"200": {"description": "Success"}}),
            }

            if doc.get("requestBody"):
                spec["paths"][path][method]["requestBody"] = doc["requestBody"]

        return spec

    @classmethod
    def _generate_swagger_html(cls, openapi_url: str) -> str:
        """Generate Swagger UI HTML"""
        return f"""
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>API Documentation - Shanks Django</title>
    <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/swagger-ui-dist@5/swagger-ui.css">
    <style>
        body {{
            margin: 0;
            padding: 0;
        }}
        .topbar {{
            display: none;
        }}
    </style>
</head>
<body>
    <div id="swagger-ui"></div>
    <script src="https://cdn.jsdelivr.net/npm/swagger-ui-dist@5/swagger-ui-bundle.js"></script>
    <script src="https://cdn.jsdelivr.net/npm/swagger-ui-dist@5/swagger-ui-standalone-preset.js"></script>
    <script>
        window.onload = function() {{
            const ui = SwaggerUIBundle({{
                url: "{openapi_url}",
                dom_id: '#swagger-ui',
                deepLinking: true,
                presets: [
                    SwaggerUIBundle.presets.apis,
                    SwaggerUIStandalonePreset
                ],
                plugins: [
                    SwaggerUIBundle.plugins.DownloadUrl
                ],
                layout: "StandaloneLayout"
            }});
            window.ui = ui;
        }};
    </script>
</body>
</html>
"""


def enable_swagger(
    app,
    title: str = "API Documentation",
    version: str = "1.0.0",
    description: str = "",
    docs_url: str = "docs",
):
    """
    Shortcut to enable Swagger UI

    Example:
        from shanks import App, enable_swagger

        app = App()
        enable_swagger(app, title="My API")
    """
    SwaggerUI.enable(app, title, version, description, docs_url)


def swagger(
    title: str = "API Documentation",
    version: str = "1.0.0",
    description: str = "",
    docs_url: str = "docs",
):
    """
    Middleware-style Swagger UI setup

    Usage:
        from shanks import App, swagger

        app = App()
        app.use(swagger(title="My API", version="1.0.0"))

        @app.get('api/users')
        def get_users(req):
            return {'users': []}
    """

    def middleware(req, res, next):
        # This is called once during app initialization
        # The actual Swagger setup happens when the middleware is added
        next()

    # Store swagger config on the middleware function
    middleware._swagger_config = {
        "title": title,
        "version": version,
        "description": description,
        "docs_url": docs_url,
    }
    middleware._is_swagger = True

    return middleware



from .app import App
from .cors import CORS, enable_cors
from .db import (
    DatabaseConfig,
    MongoDB,
    Redis,
    setup_mongodb,
    setup_mysql,
    setup_postgres,
    setup_redis,
    setup_sqlite,
)
from .request import Request
from .response import Response
from .swagger import SwaggerUI, enable_swagger, swagger
from .cache import (
    cache,
    auto_cache,
    invalidate_cache,
    smart_cache_invalidation,
    get_cache,
)

__version__ = "0.1.5"
__author__ = "Ararya"
__email__ = "araryaarch@gmail.com"


# Lazy import ORM to avoid Django settings requirement
def __getattr__(name):
    if name in [
        "Model",
        "User",
        "authenticate",
        "create_user",
        "slugify",
        "CharField",
        "TextField",
        "IntegerField",
        "FloatField",
        "BooleanField",
        "DateField",
        "DateTimeField",
        "EmailField",
        "URLField",
        "SlugField",
        "JSONField",
        "ForeignKey",
        "ManyToManyField",
        "OneToOneField",
        "CASCADE",
        "SET_NULL",
        "PROTECT",
    ]:
        from . import orm

        return getattr(orm, name)

    # Schema imports
    if name in ["Schema", "model"]:
        from . import schema

        return getattr(schema, name)

    # Config imports
    if name in [
        "Config",
        "env",
        "env_bool",
        "env_list",
        "env_int",
        "get_base_dir",
        "get_secret_key",
        "get_debug",
        "get_allowed_hosts",
        "get_database",
        "get_installed_apps",
        "get_middleware",
        "get_templates",
        "get_password_validators",
    ]:
        from . import config

        return getattr(config, name)

    raise AttributeError(f"module '{__name__}' has no attribute '{name}'")


__all__ = [
    "App",
    "Request",
    "Response",
    "CORS",
    "enable_cors",
    "SwaggerUI",
    "enable_swagger",
    "swagger",
    # Cache
    "cache",
    "auto_cache",
    "invalidate_cache",
    "smart_cache_invalidation",
    "get_cache",
    # ORM
    "Model",
    "User",
    "authenticate",
    "create_user",
    "slugify",
    "CharField",
    "TextField",
    "IntegerField",
    "FloatField",
    "BooleanField",
    "DateField",
    "DateTimeField",
    "EmailField",
    "URLField",
    "SlugField",
    "JSONField",
    "ForeignKey",
    "ManyToManyField",
    "OneToOneField",
    "CASCADE",
    "SET_NULL",
    "PROTECT",
    # Database
    "DatabaseConfig",
    "MongoDB",
    "Redis",
    "setup_postgres",
    "setup_mysql",
    "setup_sqlite",
    "setup_mongodb",
    "setup_redis",
    # Schema
    "Schema",
    "model",
    # Config
    "Config",
    "env",
    "env_bool",
    "env_list",
    "env_int",
    "get_base_dir",
    "get_secret_key",
    "get_debug",
    "get_allowed_hosts",
    "get_database",
    "get_installed_apps",
    "get_middleware",
    "get_templates",
    "get_password_validators",
]

# CLI is available via 'shanks' command after installation



"""JSON-like schema for defining models - Super simple!"""

from django.db import models


class Schema:
    """Define models with JSON-like syntax"""

    @staticmethod
    def model(name, fields, options=None):
        """
        Create a Django model from JSON-like schema

        Example:
            Post = Schema.model("Post", {
                "title": "string",
                "content": "text",
                "published": "boolean",
                "views": "integer",
                "author": {"type": "relation", "model": "User"},
                "created_at": "datetime"
            })
        """
        import inspect

        # Get the calling module to set app_label correctly
        frame = inspect.currentframe().f_back
        module = inspect.getmodule(frame)
        module_name = module.__name__ if module else "app"

        # Extract app name from module path (e.g., "entity.products" -> "entity")
        app_label = module_name.split(".")[0] if "." in module_name else module_name

        attrs = {"__module__": module_name}

        # Add Meta class
        meta_attrs = {"app_label": app_label}

        if options:
            if "table" in options:
                meta_attrs["db_table"] = options["table"]
            if "ordering" in options:
                meta_attrs["ordering"] = options["ordering"]

        attrs["Meta"] = type("Meta", (), meta_attrs)

        # Convert JSON-like fields to Django fields
        for field_name, field_def in fields.items():
            attrs[field_name] = Schema._create_field(field_def)

        # Create model class with Prisma-like methods
        model_class = type(name, (models.Model,), attrs)

        # Add Prisma-like methods
        Schema._add_prisma_methods(model_class)

        return model_class

    @staticmethod
    def _create_field(field_def):
        """Convert JSON field definition to Django field"""
        # Simple string type
        if isinstance(field_def, str):
            return Schema._parse_field_string(field_def)

        # Dict with options
        if isinstance(field_def, dict):
            field_type = field_def.get("type")
            options = {k: v for k, v in field_def.items() if k != "type"}

            # Handle relations
            if field_type == "relation":
                model = options.pop("model")
                on_delete = options.pop("on_delete", models.CASCADE)
                return models.ForeignKey(model, on_delete=on_delete, **options)

            if field_type == "many":
                model = options.pop("model")
                return models.ManyToManyField(model, **options)

            # Regular field with options
            field_class = Schema._get_field_by_type(field_type)
            return field_class(**options)

        return field_def

    @staticmethod
    def _parse_field_string(field_str):
        """
        Parse field string like 'string:100:unique' or 'number:auto_now_add'
        JavaScript-like types: string, number, boolean, date
        """
        parts = field_str.split(":")
        field_type = parts[0]
        options = {}

        # Parse options from string
        for part in parts[1:]:
            if part.isdigit():
                options["max_length"] = int(part)
            elif part == "unique":
                options["unique"] = True
            elif part == "blank":
                options["blank"] = True
            elif part == "null":
                options["null"] = True
            elif part == "auto_now":
                options["auto_now"] = True
            elif part == "auto_now_add":
                options["auto_now_add"] = True

        # Get field class
        field_class = Schema._get_field_class_by_type(field_type)

        # Create field instance
        if field_class:
            return field_class(**options)

        return models.CharField(max_length=255)

    @staticmethod
    def _get_field_class_by_type(field_type):
        """Get Django field class by JavaScript-like type string"""
        type_map = {
            # JavaScript-like types
            "string": models.CharField,
            "number": models.IntegerField,
            "boolean": models.BooleanField,
            "date": models.DateTimeField,
            # Additional types
            "text": models.TextField,
            "float": models.FloatField,
            "email": models.EmailField,
            "url": models.URLField,
            "slug": models.SlugField,
            "json": models.JSONField,
        }
        return type_map.get(field_type)

    @staticmethod
    def _get_field_by_type(field_type):
        """Get Django field class by type string (legacy)"""
        type_map = {
            "string": models.CharField(max_length=255),
            "text": models.TextField(),
            "number": models.IntegerField(),
            "integer": models.IntegerField(),
            "float": models.FloatField(),
            "boolean": models.BooleanField(default=False),
            "date": models.DateTimeField(auto_now_add=True),
            "datetime": models.DateTimeField(auto_now_add=True),
            "email": models.EmailField(),
            "url": models.URLField(),
            "slug": models.SlugField(),
            "json": models.JSONField(),
        }
        return type_map.get(field_type, models.CharField(max_length=255))

    @staticmethod
    def _add_prisma_methods(model_class):
        """Add Prisma-like methods to model"""

        @classmethod
        def find_many(cls, **filters):
            return cls.objects.filter(**filters)

        @classmethod
        def find_first(cls, **filters):
            return cls.objects.filter(**filters).first()

        @classmethod
        def find_unique(cls, **filters):
            try:
                return cls.objects.get(**filters)
            except cls.DoesNotExist:
                return None

        @classmethod
        def create(cls, **data):
            return cls.objects.create(**data)

        @classmethod
        def count(cls, **filters):
            if filters:
                return cls.objects.filter(**filters).count()
            return cls.objects.count()

        def update_self(self, **data):
            for key, value in data.items():
                setattr(self, key, value)
            self.save()
            return self

        def delete_self(self):
            self.delete()

        # Attach methods
        model_class.find_many = find_many
        model_class.find_first = find_first
        model_class.find_unique = find_unique
        model_class.create = create
        model_class.count = count
        model_class.update_self = update_self
        model_class.delete_self = delete_self


# Shorthand function
def model(name, fields, options=None):
    """Shorthand for Schema.model()"""
    return Schema.model(name, fields, options)



"""Built-in CORS support for Shanks Django"""

from typing import List, Optional, Union

from .response import Response


class CORS:
    """
    Built-in CORS middleware for Shanks Django

    Usage:
        from shanks import App, CORS

        app = App()

        # Enable CORS for all routes
        CORS.enable(app)

        # Or with custom settings
        CORS.enable(app,
            origins=['http://localhost:3000', 'https://myapp.com'],
            methods=['GET', 'POST', 'PUT', 'DELETE'],
            headers=['Content-Type', 'Authorization'],
            credentials=True
        )
    """

    @staticmethod
    def enable(
        app,
        origins: Union[str, List[str]] = "*",
        methods: Union[str, List[str]] = "*",
        headers: Union[str, List[str]] = "*",
        credentials: bool = False,
        max_age: int = 86400,
    ):
        """
        Enable CORS for the app

        Args:
            app: Shanks App instance
            origins: Allowed origins ('*' or list of origins)
            methods: Allowed methods ('*' or list of methods)
            headers: Allowed headers ('*' or list of headers)
            credentials: Allow credentials (cookies, auth headers)
            max_age: Preflight cache duration in seconds

        Example:
            # Allow all origins
            CORS.enable(app)

            # Specific origins
            CORS.enable(app, origins=['http://localhost:3000'])

            # With credentials
            CORS.enable(app,
                origins=['http://localhost:3000'],
                credentials=True
            )
        """

        def cors_middleware(req):
            # Get origin from request
            origin = req.headers.get("Origin", "")

            # Check if origin is allowed
            if origins == "*":
                allowed_origin = "*"
            elif isinstance(origins, list):
                if origin in origins:
                    allowed_origin = origin
                else:
                    allowed_origin = None
            else:
                allowed_origin = origins if origin == origins else None

            # Handle preflight OPTIONS request
            if req.method == "OPTIONS":
                response = Response().status_code(204)

                if allowed_origin:
                    response.header("Access-Control-Allow-Origin", allowed_origin)

                if credentials:
                    response.header("Access-Control-Allow-Credentials", "true")

                # Methods
                if methods == "*":
                    response.header(
                        "Access-Control-Allow-Methods",
                        "GET, POST, PUT, DELETE, PATCH, OPTIONS",
                    )
                elif isinstance(methods, list):
                    response.header("Access-Control-Allow-Methods", ", ".join(methods))
                else:
                    response.header("Access-Control-Allow-Methods", methods)

                # Headers
                if headers == "*":
                    requested_headers = req.headers.get(
                        "Access-Control-Request-Headers", ""
                    )
                    if requested_headers:
                        response.header(
                            "Access-Control-Allow-Headers", requested_headers
                        )
                    else:
                        response.header(
                            "Access-Control-Allow-Headers",
                            "Content-Type, Authorization, X-Requested-With",
                        )
                elif isinstance(headers, list):
                    response.header("Access-Control-Allow-Headers", ", ".join(headers))
                else:
                    response.header("Access-Control-Allow-Headers", headers)

                response.header("Access-Control-Max-Age", str(max_age))

                return response.to_django_response(req.django)

            # For actual requests, we'll add headers in the response wrapper
            # Store CORS config in request for later use
            req._cors_config = {
                "allowed_origin": allowed_origin,
                "credentials": credentials,
            }

        # Add CORS middleware
        app.use(cors_middleware)

        # Wrap the _create_view to add CORS headers to responses
        original_create_view = app._create_view

        def create_view_with_cors(handler, method):
            view = original_create_view(handler, method)

            def wrapped_view(request, *args, **kwargs):
                response = view(request, *args, **kwargs)

                # Add CORS headers to response
                if hasattr(request, "_shanks_request"):
                    shanks_req = request._shanks_request
                    if hasattr(shanks_req, "_cors_config"):
                        cors_config = shanks_req._cors_config
                        if cors_config["allowed_origin"]:
                            response["Access-Control-Allow-Origin"] = cors_config[
                                "allowed_origin"
                            ]
                        if cors_config["credentials"]:
                            response["Access-Control-Allow-Credentials"] = "true"

                return response

            return wrapped_view

        app._create_view = create_view_with_cors

    @staticmethod
    def middleware(
        origins: Union[str, List[str]] = "*",
        methods: Union[str, List[str]] = "*",
        headers: Union[str, List[str]] = "*",
        credentials: bool = False,
        max_age: int = 86400,
    ):
        """
        Create a CORS middleware function

        Example:
            from shanks import App, CORS

            app = App()

            # Add CORS middleware
            app.use(CORS.middleware(
                origins=['http://localhost:3000'],
                credentials=True
            ))
        """

        def cors_middleware(req):
            origin = req.headers.get("Origin", "")

            # Check if origin is allowed
            if origins == "*":
                allowed_origin = "*"
            elif isinstance(origins, list):
                allowed_origin = origin if origin in origins else None
            else:
                allowed_origin = origins if origin == origins else None

            # Handle preflight
            if req.method == "OPTIONS":
                response = Response().status_code(204)

                if allowed_origin:
                    response.header("Access-Control-Allow-Origin", allowed_origin)

                if credentials:
                    response.header("Access-Control-Allow-Credentials", "true")

                if methods == "*":
                    response.header(
                        "Access-Control-Allow-Methods",
                        "GET, POST, PUT, DELETE, PATCH, OPTIONS",
                    )
                elif isinstance(methods, list):
                    response.header("Access-Control-Allow-Methods", ", ".join(methods))

                if headers == "*":
                    requested_headers = req.headers.get(
                        "Access-Control-Request-Headers", ""
                    )
                    response.header(
                        "Access-Control-Allow-Headers",
                        requested_headers
                        or "Content-Type, Authorization, X-Requested-With",
                    )
                elif isinstance(headers, list):
                    response.header("Access-Control-Allow-Headers", ", ".join(headers))

                response.header("Access-Control-Max-Age", str(max_age))
                return response

            # Store for actual requests
            req._cors_origin = allowed_origin
            req._cors_credentials = credentials

        return cors_middleware


def enable_cors(
    app,
    origins: Union[str, List[str]] = "*",
    methods: Union[str, List[str]] = "*",
    headers: Union[str, List[str]] = "*",
    credentials: bool = False,
    max_age: int = 86400,
):
    """
    Shortcut function to enable CORS

    Example:
        from shanks import App, enable_cors

        app = App()
        enable_cors(app)
    """
    CORS.enable(app, origins, methods, headers, credentials, max_age)



from django.http import HttpResponse, HttpResponseRedirect, JsonResponse
from django.shortcuts import render as django_render


class Response:
    """Express-like response builder"""

    def __init__(self, data=None, status=200):
        self.data = data
        self.status = status
        self._headers = {}
        self._cookies = []
        self._template = None
        self._context = {}

    def json(self, data):
        """Send JSON response"""
        self.data = data
        return self

    def status_code(self, code):
        """Set status code"""
        self.status = code
        return self

    def header(self, key, value):
        """Set response header"""
        self._headers[key] = value
        return self

    def cookie(self, key, value, **options):
        """Set cookie"""
        self._cookies.append((key, value, options))
        return self

    def redirect(self, url):
        """Redirect to URL"""
        self.data = url
        self._template = "redirect"
        return self

    def render(self, request, template, context=None):
        """Render Django template"""
        self._template = template
        self._context = context or {}
        self._context["request"] = request
        return self

    def to_django_response(self, request=None):
        """Convert to Django response"""
        # Handle redirect
        if self._template == "redirect":
            response = HttpResponseRedirect(self.data)
        # Handle template rendering
        elif self._template and self._template != "redirect":
            response = django_render(request, self._template, self._context)
            response.status_code = self.status
        # Handle JSON
        elif isinstance(self.data, dict):
            response = JsonResponse(self.data, status=self.status)
        # Handle plain text/HTML
        else:
            response = HttpResponse(self.data, status=self.status)

        # Set headers
        for key, value in self._headers.items():
            response[key] = value

        # Set cookies
        for key, value, options in self._cookies:
            response.set_cookie(key, value, **options)

        return response



"""Basic tests for Shanks Django"""

import json

from django.test import RequestFactory

from shanks import App, Request, Response


def test_shanks_import():
    """Test that Shanks can be imported"""
    assert App is not None


def test_simple_route():
    """Test simple GET route"""
    app = App()

    @app.get("api/test")
    def test_view(req):
        return {"status": "ok"}

    assert len(app.routes) == 1
    assert app.routes[0]["path"] == "api/test"


def test_request_wrapper():
    """Test Request wrapper"""
    factory = RequestFactory()
    django_req = factory.get("/test?name=john")

    req = Request(django_req)
    assert req.method == "GET"
    assert req.path == "/test"
    # query returns a dict from QueryDict
    query_dict = req.query
    assert query_dict["name"] == "john"


def test_response_builder():
    """Test Response builder"""
    resp = Response().status_code(201).json({"created": True})

    assert resp.status == 201
    assert resp.data == {"created": True}


def test_middleware():
    """Test middleware support"""
    app = App()

    called = []

    def middleware(req):
        called.append(True)

    app.use(middleware)

    @app.get("test")
    def view(req):
        return {"ok": True}

    assert len(app.middlewares) == 1



"""Configuration helpers for Shanks Django"""

import os
from pathlib import Path


class Config:
    """Auto-load environment variables and provide helpers"""

    _loaded = False

    @classmethod
    def load(cls):
        """Auto-load .env file"""
        if cls._loaded:
            return

        try:
            from dotenv import load_dotenv

            load_dotenv()
            cls._loaded = True
        except ImportError:
            # dotenv not installed, skip
            pass

    @classmethod
    def get(cls, key, default=None):
        """Get environment variable with auto-load"""
        cls.load()
        return os.getenv(key, default)

    @classmethod
    def get_bool(cls, key, default=False):
        """Get boolean environment variable"""
        cls.load()
        value = os.getenv(key, str(default))
        return value.lower() in ("true", "1", "yes", "on")

    @classmethod
    def get_list(cls, key, default=None, separator=","):
        """Get list from environment variable"""
        cls.load()
        value = os.getenv(key)
        if value is None:
            return default or []
        return [item.strip() for item in value.split(separator)]

    @classmethod
    def get_int(cls, key, default=0):
        """Get integer environment variable"""
        cls.load()
        value = os.getenv(key)
        if value is None:
            return default
        try:
            return int(value)
        except ValueError:
            return default


def env(key, default=None):
    """Shorthand for Config.get()"""
    return Config.get(key, default)


def env_bool(key, default=False):
    """Shorthand for Config.get_bool()"""
    return Config.get_bool(key, default)


def env_list(key, default=None, separator=","):
    """Shorthand for Config.get_list()"""
    return Config.get_list(key, default, separator)


def env_int(key, default=0):
    """Shorthand for Config.get_int()"""
    return Config.get_int(key, default)


# Auto-load on import
Config.load()


def get_base_dir(file_path):
    """Get base directory from __file__"""
    return Path(file_path).resolve().parent


def get_secret_key(default="django-insecure-change-this-in-production"):
    """Get SECRET_KEY from environment"""
    return env("SECRET_KEY", default)


def get_debug(default=True):
    """Get DEBUG from environment"""
    return env_bool("DEBUG", default)


def get_allowed_hosts(default="*"):
    """Get ALLOWED_HOSTS from environment"""
    return env_list("ALLOWED_HOSTS", [default])


def get_database(base_dir):
    """Get database configuration"""
    database_url = env("DATABASE_URL")

    if database_url:
        from .db import DatabaseConfig

        return {"default": DatabaseConfig.from_url(database_url)}

    return {
        "default": {
            "ENGINE": "django.db.backends.sqlite3",
            "NAME": base_dir / "db.sqlite3",
        }
    }


def get_installed_apps(extra_apps=None):
    """Get default INSTALLED_APPS"""
    apps = [
        "django.contrib.admin",
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "django.contrib.sessions",
        "django.contrib.messages",
        "django.contrib.staticfiles",
    ]

    if extra_apps:
        apps.extend(extra_apps)

    return apps


def get_middleware():
    """Get default MIDDLEWARE"""
    return [
        "django.middleware.security.SecurityMiddleware",
        "django.contrib.sessions.middleware.SessionMiddleware",
        "django.middleware.common.CommonMiddleware",
        "django.contrib.auth.middleware.AuthenticationMiddleware",
        "django.contrib.messages.middleware.MessageMiddleware",
        "django.middleware.clickjacking.XFrameOptionsMiddleware",
    ]


def get_templates():
    """Get default TEMPLATES configuration"""
    return [
        {
            "BACKEND": "django.template.backends.django.DjangoTemplates",
            "DIRS": [],
            "APP_DIRS": True,
            "OPTIONS": {
                "context_processors": [
                    "django.template.context_processors.debug",
                    "django.template.context_processors.request",
                    "django.contrib.auth.context_processors.auth",
                    "django.contrib.messages.context_processors.messages",
                ],
            },
        },
    ]


def get_password_validators(debug=True):
    """Get password validators (disabled in debug mode)"""
    if debug:
        return []

    return [
        {
            "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator"
        },
        {"NAME": "django.contrib.auth.password_validation.MinimumLengthValidator"},
        {"NAME": "django.contrib.auth.password_validation.CommonPasswordValidator"},
        {"NAME": "django.contrib.auth.password_validation.NumericPasswordValidator"},
    ]



"""Built-in caching for Shanks - Auto-cache GET requests"""

import hashlib
import json
import time
from functools import wraps


class SimpleCache:
    """Simple in-memory cache with TTL"""

    def __init__(self):
        self._cache = {}
        self._timestamps = {}

    def get(self, key):
        """Get value from cache if not expired"""
        if key in self._cache:
            timestamp = self._timestamps.get(key, 0)
            if time.time() - timestamp < self._cache[key]["ttl"]:
                return self._cache[key]["value"]
            else:
                # Expired, remove
                del self._cache[key]
                del self._timestamps[key]
        return None

    def set(self, key, value, ttl=300):
        """Set value in cache with TTL (default 5 minutes)"""
        self._cache[key] = {"value": value, "ttl": ttl}
        self._timestamps[key] = time.time()

    def delete(self, key):
        """Delete key from cache"""
        if key in self._cache:
            del self._cache[key]
            del self._timestamps[key]

    def clear(self):
        """Clear all cache"""
        self._cache.clear()
        self._timestamps.clear()

    def invalidate_pattern(self, pattern):
        """Invalidate all keys matching pattern"""
        keys_to_delete = [k for k in self._cache.keys() if pattern in k]
        for key in keys_to_delete:
            self.delete(key)


# Global cache instance
_cache = SimpleCache()


def get_cache():
    """Get global cache instance"""
    return _cache


def cache_key(request):
    """Generate cache key from request"""
    # Include method, path, and query params
    key_parts = [
        request.method,
        request.path,
        json.dumps(dict(request.GET), sort_keys=True),
    ]
    key_string = "|".join(key_parts)
    return hashlib.md5(key_string.encode()).hexdigest()


def cache(ttl=300, methods=None):
    """
    Decorator to cache endpoint responses

    Args:
        ttl: Time to live in seconds (default 5 minutes)
        methods: List of HTTP methods to cache (default: ['GET'])

    Example:
        @app.get("api/posts")
        @cache(ttl=600)  # Cache for 10 minutes
        def list_posts(req):
            return {"posts": [...]}
    """
    if methods is None:
        methods = ["GET"]

    def decorator(func):
        @wraps(func)
        def wrapper(request, *args, **kwargs):
            # Only cache specified methods
            if request.method not in methods:
                return func(request, *args, **kwargs)

            # Generate cache key
            key = cache_key(request)

            # Try to get from cache
            cached = _cache.get(key)
            if cached is not None:
                return cached

            # Execute function
            response = func(request, *args, **kwargs)

            # Cache the response
            _cache.set(key, response, ttl)

            return response

        return wrapper

    return decorator


def auto_cache(req, res, next):
    """
    Middleware for automatic caching of GET requests

    Usage in routes/__init__.py:
        from shanks import auto_cache
        app.use(auto_cache)
    """
    # Only cache GET requests
    if req.method != "GET":
        next()
        return

    # Generate cache key
    key = cache_key(req)

    # Try to get from cache
    cached = _cache.get(key)
    if cached is not None:
        # Add cache header
        if hasattr(res, "headers"):
            res.headers["X-Cache"] = "HIT"
        return cached

    # Store original response
    original_response = [None]

    def cache_response():
        """Cache the response after next() completes"""
        if original_response[0] is not None:
            _cache.set(key, original_response[0], ttl=300)  # 5 minutes default

    # Wrap next to capture response
    def wrapped_next():
        result = next()
        original_response[0] = result
        cache_response()
        return result

    return wrapped_next()


def invalidate_cache(pattern=None):
    """
    Invalidate cache entries

    Args:
        pattern: Pattern to match (None = clear all)

    Example:
        # Clear all cache
        invalidate_cache()

        # Clear specific pattern
        invalidate_cache("/api/posts")
    """
    if pattern is None:
        _cache.clear()
    else:
        _cache.invalidate_pattern(pattern)


# Smart cache invalidation middleware
def smart_cache_invalidation(req, res, next):
    """
    Middleware to auto-invalidate cache on POST/PUT/DELETE

    Usage:
        app.use(smart_cache_invalidation)
    """
    # Invalidate cache for write operations
    if req.method in ["POST", "PUT", "PATCH", "DELETE"]:
        # Extract resource path (e.g., /api/posts/123 -> /api/posts)
        path_parts = req.path.split("/")
        if len(path_parts) > 2:
            resource_path = (
                "/".join(path_parts[:-1]) if path_parts[-1].isdigit() else req.path
            )
            invalidate_cache(resource_path)

    next()


__all__ = [
    "cache",
    "auto_cache",
    "invalidate_cache",
    "smart_cache_invalidation",
    "get_cache",
]



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="shanks-django",
    version="0.1.0",
    author="Ararya",
    author_email="araryaarch@gmail.com",
    description="Express.js-like framework built on Django",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Ararya/shanks-django",
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*", "example-project", "docs-website", "vscode-extension"]),
    include_package_data=True,
    package_data={
        "shanks": ["templates/*.html"],
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Framework :: Django",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.0",
        "Framework :: Django :: 4.1",
        "Framework :: Django :: 4.2",
        "Framework :: Django :: 5.0",
    ],
    python_requires=">=3.8",
    install_requires=[
        "django>=3.2",
        "black>=23.0.0",
        "watchdog>=3.0.0",
    ],
    extras_require={
        "postgres": ["psycopg2-binary>=2.9.0"],
        "mysql": ["mysqlclient>=2.1.0"],
        "mongodb": ["pymongo>=4.0.0"],
        "redis": ["redis>=4.0.0"],
        "all": [
            "psycopg2-binary>=2.9.0",
            "mysqlclient>=2.1.0",
            "pymongo>=4.0.0",
            "redis>=4.0.0",
            "dj-database-url>=2.0.0",
        ],
    },
    entry_points={
        "console_scripts": [
            "shanks=shanks.cli:main",
        ],
    },
    keywords="django express framework web api rest",
    project_urls={
        "Bug Reports": "https://github.com/Ararya/shanks-django/issues",
        "Source": "https://github.com/Ararya/shanks-django",
        "Documentation": "https://github.com/Ararya/shanks-django#readme",
    },
)
